package routes_test

import (
	"bytes"
	"gitea.deepak.science/deepak/gogmagog/models"
	"gitea.deepak.science/deepak/gogmagog/routes"
	"gitea.deepak.science/deepak/gogmagog/tokens"
	"github.com/stretchr/testify/assert"
	"net/http"
	"net/http/httptest"
	"testing"
)

func TestLoginAuth(t *testing.T) {
	// set up
	assert := assert.New(t)

	m := getEmptyModel()
	username := "testing_username"
	displayName := "testing_name"
	password := "pass"
	m.CreateUser(&models.CreateUserRequest{Username: username, DisplayName: displayName, Password: password})

	toker := tokens.GetDeterministicToker()
	data := []byte(`{
			"username": "testing_username",
			"password": "pass"
		}`)
	req, _ := http.NewRequest("POST", "/tokens", bytes.NewBuffer(data))

	rr := httptest.NewRecorder()

	// function under test
	router := routes.NewAuthRouter(m, toker)
	router.ServeHTTP(rr, req)

	// check results
	status := rr.Code
	assert.Equal(http.StatusOK, status)
	expected := `{
		"token": "{\"ID\":1,\"Username\":\"testing_username\"}"
	}`
	assert.JSONEq(expected, rr.Body.String())
	contentType := rr.Header().Get("Content-Type")
	assert.Equal("application/json", contentType)

}

func TestLoginBadCreds(t *testing.T) {
	// set up
	assert := assert.New(t)
	m := getEmptyModel()
	toker := tokens.GetDeterministicToker()
	data := []byte(`{
			"username": "testing_use
		}`)
	req, _ := http.NewRequest("POST", "/tokens", bytes.NewBuffer(data))

	rr := httptest.NewRecorder()

	// function under test
	router := routes.NewAuthRouter(m, toker)
	router.ServeHTTP(rr, req)

	// check results
	status := rr.Code
	assert.Equal(http.StatusBadRequest, status)

}

func TestLoginBadRequestTwoBodies(t *testing.T) {
	// set up
	assert := assert.New(t)
	m := getEmptyModel()
	toker := tokens.GetDeterministicToker()
	data := []byte(`{
			"username": "testing_username",
			"password": "pass"
		}{
				"username": "testing_username",
				"password": "pass"
			}`)
	req, _ := http.NewRequest("POST", "/tokens", bytes.NewBuffer(data))

	rr := httptest.NewRecorder()

	// function under test
	router := routes.NewAuthRouter(m, toker)
	router.ServeHTTP(rr, req)

	// check results
	status := rr.Code
	assert.Equal(http.StatusBadRequest, status)
}

func TestLoginAuthWrongPass(t *testing.T) {
	// set up
	assert := assert.New(t)

	m := getEmptyModel()
	username := "testing_username"
	displayName := "testing_name"
	password := "pass"
	m.CreateUser(&models.CreateUserRequest{Username: username, DisplayName: displayName, Password: password})

	toker := tokens.GetDeterministicToker()
	data := []byte(`{
			"username": "testing_username",
			"password": "badpass"
		}`)
	req, _ := http.NewRequest("POST", "/tokens", bytes.NewBuffer(data))

	rr := httptest.NewRecorder()

	// function under test
	router := routes.NewAuthRouter(m, toker)
	router.ServeHTTP(rr, req)

	// check results
	status := rr.Code
	assert.Equal(http.StatusUnauthorized, status)

}

func TestLoginErrorModel(t *testing.T) {
	// set up
	assert := assert.New(t)

	m := getErrorModel("error")

	toker := tokens.GetDeterministicToker()
	data := []byte(`{
			"username": "testing_username",
			"password": "badpass"
		}`)
	req, _ := http.NewRequest("POST", "/tokens", bytes.NewBuffer(data))

	rr := httptest.NewRecorder()

	// function under test
	router := routes.NewAuthRouter(m, toker)
	router.ServeHTTP(rr, req)

	// check results
	status := rr.Code
	assert.Equal(http.StatusInternalServerError, status)

}

func TestLoginBadWriter(t *testing.T) {
	// set up
	assert := assert.New(t)

	m := getEmptyModel()
	username := "testing_username"
	displayName := "testing_name"
	password := "pass"
	m.CreateUser(&models.CreateUserRequest{Username: username, DisplayName: displayName, Password: password})

	toker := tokens.GetDeterministicToker()
	data := []byte(`{
			"username": "testing_username",
			"password": "pass"
		}`)
	req, _ := http.NewRequest("POST", "/tokens", bytes.NewBuffer(data))

	rr := NewBadWriter()

	// function under test
	router := routes.NewAuthRouter(m, toker)
	router.ServeHTTP(rr, req)

	// check results
	status := rr.Code
	assert.Equal(http.StatusInternalServerError, status)

}

//
// func TestRegisterBadWriter(t *testing.T) {
// 	// set up
// 	assert := assert.New(t)
// 	m := getEmptyModel()
// 	toker := tokens.New("secret")
// 	data := []byte(`{
// 			"username": "test",
// 			"password": "pass",
// 			"display_name": "My Display Name"
// 		}`)
// 	req, _ := http.NewRequest("POST", "/register", bytes.NewBuffer(data))
//
// 	rr := NewBadWriter()
//
// 	// function under test
// 	router := routes.NewAuthRouter(m, toker)
// 	router.ServeHTTP(rr, req)
//
// 	// check results
// 	status := rr.Code
// 	assert.Equal(http.StatusInternalServerError, status)
//
// }
